@@ -23,10 +23,12 @@
 
 import logging
 import json
-import requests
+from requests.sessions import Session
+
 import botocore.auth
 import botocore.response
 import botocore.exceptions
+from botocore.awsrequest import AWSRequest
 
 try:
     from urllib.parse import urljoin
@@ -47,26 +49,26 @@ class Endpoint(object):
     :ivar session: The session object.
     """
 
-    def __init__(self, service, region_name, host):
+    def __init__(self, service, region_name, host, auth):
         self.service = service
         self.session = self.service.session
         self.region_name = region_name
         self.host = host
         self.verify = True
-        if hasattr(self.service, 'signing_name'):
-            signing_name = self.service.signing_name
-        else:
-            signing_name = self.service.endpoint_prefix
-        self.auth = botocore.auth.get_auth(self.service.signature_version,
-                                           credentials=self.session.get_credentials(),
-                                           service_name=signing_name,
-                                           region_name=region_name)
+        self.auth = auth
+        self.http_session = Session()
 
     def __repr__(self):
-        return '%s(%s)' % (self.service.name, self.host)
+        return '%s(%s)' % (self.service.endpoint_prefix, self.host)
 
     def make_request(self, params, list_marker=None):
-        pass
+        raise NotImplementedError("make_request")
+
+    def prepare_request(self, request):
+        logger.debug('prepare_request')
+        self.auth.add_auth(request=request)
+        prepared_request = request.prepare()
+        return prepared_request
 
 
 class QueryEndpoint(Endpoint):
@@ -85,13 +87,14 @@ class QueryEndpoint(Endpoint):
         params['Action'] = operation.name
         params['Version'] = self.service.api_version
         user_agent = self.session.user_agent()
-        http_response = requests.post(self.host, params=params,
-                                      hooks={'args': self.auth.add_auth},
-                                      headers={'User-Agent': user_agent},
-                                      verify=self.verify)
+        request = AWSRequest(method='POST', url=self.host,
+                             data=params, headers={'User-Agent': user_agent})
+        prepared_request = self.prepare_request(request)
+        http_response = self.http_session.send(prepared_request,
+                                               verify=self.verify)
         r = botocore.response.Response(operation)
         http_response.encoding = 'utf-8'
-        body = http_response.text.encode('utf=8')
+        body = http_response.text.encode('utf-8')
         logger.debug(body)
         r.parse(body)
         return (http_response, r.get_value())
@@ -102,6 +105,9 @@ class JSONEndpoint(Endpoint):
     This class handles only AWS/JSON style services.
     """
 
+    ResponseContentTypes = ['application/x-amz-json-1.1',
+                            'application/json']
+
     def make_request(self, operation, params):
         """
         Send a request to the endpoint and parse the response
@@ -112,23 +118,35 @@ class JSONEndpoint(Endpoint):
         logger.debug('SSL Verify: %s' % self.verify)
         user_agent = self.session.user_agent()
         target = '%s.%s' % (self.service.target_prefix, operation.name)
-        content_type = 'application/x-amz-json-1.1'
+        json_version = '1.0'
+        if hasattr(self.service, 'json_version'):
+            json_version = str(self.service.json_version)
+        content_type = 'application/x-amz-json-%s' % json_version
+        content_encoding = 'amz-1.0'
         data = json.dumps(params)
-        http_response = requests.post(self.host, data=data,
-                                      hooks={'args': self.auth.add_auth},
-                                      headers={'User-Agent': user_agent,
-                                               'X-Amz-Target': target,
-                                               'Content-Type': content_type},
-                                      verify=self.verify)
+        request = AWSRequest(method='POST', url=self.host,
+                             data=data,
+                             headers={'User-Agent': user_agent,
+                                      'X-Amz-Target': target,
+                                      'Content-Type': content_type,
+                                      'Content-Encoding': content_encoding})
+        prepared_request = self.prepare_request(request)
+        http_response = self.http_session.send(prepared_request,
+                                               verify=self.verify)
         http_response.encoding = 'utf-8'
-        body = http_response.text.encode('utf=8')
+        body = http_response.text.encode('utf-8')
+        logger.debug(http_response.headers)
         logger.debug(body)
-        return (http_response, json.loads(body))
+        try:
+            body = json.loads(body)
+        except:
+            pass
+        return (http_response, body)
 
 
 class RestXMLEndpoint(Endpoint):
 
-    def get_path_and_query_params(self, operation, params):
+    def build_uri(self, operation, params):
         uri = operation.http['uri']
         if '?' in uri:
             path, query_params = uri.split('?')
@@ -143,16 +161,26 @@ class RestXMLEndpoint(Endpoint):
                 pc = pc.format(**params['uri_params'])
             path_components.append(pc)
         path = '/'.join(path_components)
-        query_param_components = {}
+        query_param_components = []
         for qpc in query_params.split('&'):
             if qpc:
-                key_name, value_name = qpc.split('=')
-                value_name = value_name.strip('{}')
-                if value_name in params['uri_params']:
-                    query_param_components[key_name] = params['uri_params'][value_name]
+                if '=' in qpc:
+                    key_name, value_name = qpc.split('=')
+                else:
+                    key_name = qpc
+                    value_name = None
+                if value_name:
+                    value_name = value_name.strip('{}')
+                    if value_name in params['uri_params']:
+                        value = params['uri_params'][value_name]
+                        query_param_components.append('%s=%s' % (key_name,
+                                                                 value))
+                else:
+                    query_param_components.append(key_name)
+        query_params = '&'.join(query_param_components)
         logger.debug('path: %s' % path)
-        logger.debug('query_params: %s' % query_param_components)
-        return path, query_param_components
+        logger.debug('query_params: %s' % query_params)
+        return path + '?' + query_params
 
     def make_request(self, operation, params):
         """
@@ -163,26 +191,42 @@ class RestXMLEndpoint(Endpoint):
         logger.debug(params)
         logger.debug('SSL Verify: %s' % self.verify)
         user_agent = self.session.user_agent()
-        request_method = getattr(requests, operation.http['method'].lower())
-        path, query_params = self.get_path_and_query_params(operation, params)
-        uri = urljoin(self.host, path)
-        http_response = request_method(uri, params=query_params,
-                                       hooks={'args': self.auth.add_auth},
-                                       headers={'User-Agent': user_agent},
-                                       verify=self.verify)
-        r = botocore.response.Response(operation)
+        params['headers']['User-Agent'] = user_agent
+        uri = self.build_uri(operation, params)
+        uri = urljoin(self.host, uri)
+        request = AWSRequest(method=operation.http['method'],
+                             url=uri, headers=params['headers'],
+                             data=params['payload'])
+        prepared_request = self.prepare_request(request)
+        http_response = self.http_session.send(prepared_request,
+                                               verify=self.verify)
         http_response.encoding = 'utf-8'
-        body = http_response.text.encode('utf=8')
+        body = http_response.text.encode('utf-8')
+        r = botocore.response.Response(operation)
         logger.debug(body)
-        r.parse(body)
+        if body:
+            r.parse(body)
         return (http_response, r.get_value())
 
 
 def get_endpoint(service, region_name, endpoint_url):
-    if service.type == 'query':
-        return QueryEndpoint(service, region_name, endpoint_url)
-    if service.type == 'json':
-        return JSONEndpoint(service, region_name, endpoint_url)
-    if service.type == 'rest-xml':
-        return RestXMLEndpoint(service, region_name, endpoint_url)
-    raise botocore.exceptions.UnknownServiceStyle(service_style=service.type)
+    service_to_endpoint = {
+        'query': QueryEndpoint,
+        'json': JSONEndpoint,
+        'rest-xml': RestXMLEndpoint,
+        'rest-json': None,
+    }
+    service_type = service.type
+    if service_type not in service_to_endpoint:
+        raise botocore.exceptions.UnknownServiceStyle(
+            service_style=service.type)
+    cls = service_to_endpoint.get(service_type)
+    if cls is None:
+        raise NotImplementedError("%s service type is not yet implemented" %
+                                  service_type)
+    service_name = getattr(service, 'signing_name', service.endpoint_prefix)
+    auth = botocore.auth.get_auth(service.signature_version,
+                                  credentials=service.session.get_credentials(),
+                                  service_name=service_name,
+                                  region_name=region_name)
+    return cls(service, region_name, endpoint_url, auth=auth)
