@@ -35,12 +35,11 @@ class Parameter(BotoCoreObject):
         self.xmlname = None
         self.required = False
         self.flattened = False
+        self.allow_file = False
         self.min = None
         self.max = None
         BotoCoreObject.__init__(self, **kwargs)
         self.cli_name = '--' + self.cli_name
-        # if not self.xmlname:
-        #     self.xmlname = self.name
         self.handle_subtypes()
 
     def handle_subtypes(self):
@@ -49,19 +48,19 @@ class Parameter(BotoCoreObject):
     def validate(self, value):
         pass
 
-    def get_label(self, label=None):
-        if not label:
-            if self.xmlname:
-                label = self.xmlname
-            else:
-                label = self.name
+    def get_label(self):
+        if self.xmlname:
+            label = self.xmlname
+        else:
+            label = self.name
         return label
 
-    def build_parameter_query(self, value, built_params,
-                              label='', flattened=False):
-        logger.debug('label=%s' % label)
+    def build_parameter_query(self, value, built_params, label=''):
         value = self.validate(value)
-        label = self.get_label(label)
+        if label:
+            label = label.format(label=self.get_label())
+        else:
+            label = self.get_label()
         built_params[label] = str(value)
 
     def build_parameter_json(self, value, built_params,
@@ -70,19 +69,18 @@ class Parameter(BotoCoreObject):
         if isinstance(built_params, list):
             built_params.append(value)
         else:
-            label = self.get_label(label)
+            label = self.get_label()
             built_params[label] = value
 
     def build_parameter_rest(self, value, built_params,
                              label=''):
-        logger.debug('value=%s' % value)
-        logger.debug(built_params)
-        logger.debug(label)
-        logger.debug(self.location)
-        if self.location == 'uri':
-            built_params['uri_params'][self.name] = value
-        elif self.location == 'header':
-            built_params['headers'][self.name] = value
+        if hasattr(self, 'location'):
+            if self.location == 'uri':
+                built_params['uri_params'][self.name] = value
+            elif self.location == 'header':
+                built_params['headers'][self.name] = value
+        else:
+            built_params['payload'] = value
 
     def build_parameter(self, style, value, built_params, label=''):
         if style == 'query':
@@ -145,14 +143,16 @@ class BooleanParameter(Parameter):
         except ValueError:
             raise ValidationError(value=str(value), type_name='boolean')
 
-    def build_parameter_query(self, value, built_params,
-                              label='', flattened=False):
+    def build_parameter_query(self, value, built_params, label=''):
         value = self.validate(value)
         if value:
             value = 'true'
         else:
             value = 'false'
-        label = self.get_label(label)
+        if label:
+            label = label.format(label=self.get_label())
+        else:
+            label = self.get_label()
         built_params[label] = value
 
     def build_parameter_json(self, value, built_params, label=''):
@@ -164,7 +164,7 @@ class BooleanParameter(Parameter):
         if isinstance(built_params, list):
             built_params.append(value)
         else:
-            label = self.get_label(label)
+            label = label + self.get_label()
             built_params[label] = value
 
 
@@ -209,42 +209,40 @@ class ListParameter(Parameter):
         if self.members:
             self.members = get_parameter(None, self.members)
 
-    def build_parameter_query(self, value, built_params,
-                              label='', flattened=False):
+    def build_parameter_query(self, value, built_params, label=''):
+        logger.debug('name: %s' % self.get_label())
+        logger.debug('label: %s' % label)
         value = self.validate(value)
-        label = self.get_label(label)
-        member_type = self.members
         # If this is not a flattened list, find the label for member
         # items in the list.
-        if self.flattened or flattened:
-            if not self.xmlname:
-                label = member_type.get_label()
-        else:
+        member_type = self.members
+        if self.flattened:
             if member_type.xmlname:
-                member_name = member_type.xmlname
+                if label:
+                    label = label.format(label=member_type.xmlname)
+                else:
+                    label = member_type.xmlname
             else:
-                member_name = 'member'
-            label = '%s.%s' % (self.name, member_name)
+                if label:
+                    label = label.format(label=self.get_label)
+                else:
+                    label = self.get_label()
+        else:
+            if label:
+                label.format(label=self.get_label())
+            else:
+                label = self.get_label()
+            label = '%s.%s' % (label, 'member')
         for i, v in enumerate(value, 1):
             member_type.build_parameter_query(v, built_params,
-                                              '%s.%d' % (label, i),
-                                              self.flattened or flattened)
+                                              '%s.%d' % (label, i))
 
     def build_parameter_json(self, value, built_params, label=''):
         value = self.validate(value)
-        label = self.get_label(label)
-        member_type = self.members
-        if self.flattened:
-            label = member_type.xmlname
-        else:
-            if member_type.xmlname:
-                member_name = member_type.xmlname
-            else:
-                member_name = 'member'
-            label = '%s.%s' % (self.name, member_name)
+        label = self.get_label()
         built_params[label] = []
-        for i, v in enumerate(value, 1):
-            member_type.build_parameter_json(v, built_params[label], None)
+        for v in value:
+            self.members.build_parameter_json(v, built_params[label], None)
 
 
 class MapParameter(Parameter):
@@ -259,24 +257,19 @@ class MapParameter(Parameter):
         if self.keys:
             self.keys = get_parameter(None, self.keys)
 
-    def build_parameter_query(self, value, built_params,
-                              label='', flattened=False):
-        logger.debug(value)
-        logger.debug('label=%s' % label)
-        logger.debug('flattened=%s' % flattened)
-        label = self.get_label(label)
+    def build_parameter_query(self, value, built_params, label=''):
+        label = self.get_label()
         key_type = self.keys
         member_type = self.members
         for i, v in enumerate(value, 1):
             built_params['%s.%d.%s' % (label, i, key_type.xmlname)] = v
             member_type.build_parameter_query(value[v], built_params,
-                                              '%s.%d.%s' % (label, i, member_type.xmlname),
-                                              flattened)
+                                              '%s.%d.%s' % (label, i, member_type.xmlname))
 
     def build_parameter_json(self, value, built_params, label=''):
         if not isinstance(value, (list, tuple)):
             value = [value]
-        label = self.get_label(label)
+        label = self.get_label()
         member_type = self.members
         for i, v in enumerate(value, 1):
             member_type.build_parameter_json(v, built_params,
@@ -296,24 +289,28 @@ class StructParameter(Parameter):
                 l.append(get_parameter(name, data))
             self.members = l
 
-    def build_parameter_query(self, value, built_params,
-                              label='', flattened=False):
-        label = self.get_label(label)
-        logger.debug('label=%s' % label)
+    def build_parameter_query(self, value, built_params, label=''):
+        if label:
+            label = label.format(label=self.get_label())
+        else:
+            label = self.get_label()
+        label = '%s.{label}' % label
         for member in self.members:
             if member.required and member.py_name not in value:
                 msg = 'Expected: %s, Got: %s' % (member.py_name, value.keys())
                 raise ValueError(msg)
             if member.py_name in value:
-                member_label = member.get_label()
                 member.build_parameter_query(value[member.py_name],
                                              built_params,
-                                             label + '.' + member_label,
-                                             flattened)
+                                             label)
 
     def build_parameter_json(self, value, built_params, label=''):
-        label = self.get_label(label)
-        built_params[label] = value
+        label = self.get_label()
+        new_value = {}
+        if isinstance(built_params, list):
+            built_params.append(new_value)
+        else:
+            built_params[label] = new_value
         for member in self.members:
             if member.required and member.py_name not in value:
                 msg = 'Expected: %s, Got: %s' % (member.py_name, value.keys())
@@ -321,7 +318,7 @@ class StructParameter(Parameter):
             if member.py_name in value:
                 member_label = member.get_label()
                 member.build_parameter_json(value[member.py_name],
-                                            built_params[label],
+                                            new_value,
                                             member_label)
 
 type_map = {
