@@ -40,7 +40,6 @@ class Service(object):
 
     def __init__(self, session, provider_name, service_name,
                  path='/', port=None):
-        self.membered_lists = False
         self.global_endpoint = None
         sdata = session.get_service_data(service_name, provider_name)
         self.__dict__.update(sdata)
@@ -57,7 +56,7 @@ class Service(object):
             setattr(self, op.py_name, op)
 
     def __repr__(self):
-        return 'Service(%s)' % self.service_abbreviation
+        return 'Service(%s)' % self.endpoint_prefix
 
     @property
     def region_names(self):
